clear
clc
close all

% add path to fast ML method
addpath ../util
% Setup
nData = 100;
maxModelOrder = 10;
nMc = 250;
pitchBounds = [1/nData;1-1/nData];
% generate the signal
trueModelOrder = 10;
trueAmplitudes = ones(trueModelOrder,1);
truePhases = 2*pi*rand(trueModelOrder,1);
complexWeight = trueAmplitudes.*exp(1i*truePhases);
trueFreq = 3*pi*2/nData;
complexSinusoids = exp(1i*trueFreq*(0:nData-1)'*(1:trueModelOrder));
noiseFreeSignal = complexSinusoids*complexWeight;
snrList = (-10:1:10)';
nSnrs = length(snrList);
noiseVariances = (noiseFreeSignal'*noiseFreeSignal)/nData*...
    10.^(-snrList/10);

% compute the CRLB for the freqeuncy (in radians/sample)
crlb = 6*noiseVariances/(nData*(nData^2-1)*...
    sum(trueAmplitudes.^2.*(1:trueModelOrder)'.^2));

% accuracy of the grid+refinement
refSnr = 30; % dB
accuracy = sqrt(10^(-refSnr/10)*6/(nData*(nData^2-1)*maxModelOrder^2)); % best-case CRLB @ 30 dB
dbGainList = [10, 3, 1, 0.1, 0.01, 0.001];
gainList = 10.^(dbGainList/10);
gridSizeList = [2*sqrt(12*(gainList-1)./gainList)/(nData*maxModelOrder),...
    accuracy];
nGridSizes = length(gridSizeList);

% run the Monte Carlo Simulation
estimationError = nan(nMc, nSnrs, nGridSizes);
computationTime = nan(nMc, nSnrs, nGridSizes);
% noiseVariances = 
nGridPoints = nan(1, nGridSizes);
for iSnr = 1:nSnrs
    snrList(iSnr)
    for jMc = 1:nMc
        noise = sqrt(noiseVariances(iSnr)/2)*randn(nData,2)*[1;1i];
        data = noiseFreeSignal+noise;
        for kGridSize = 1:nGridSizes
            nGridPoints(kGridSize) = ceil(2*pi/gridSizeList(kGridSize));
            % compute the estimate on a grid
            tic
            [costFunctionMatrix, fullPitchGrid] = ...
                singlePitchMlComplexCostFunction(data,maxModelOrder, ...
                    gridSizeList(kGridSize)/(2*pi),pitchBounds);
            [~,idx] = max(costFunctionMatrix,[],2);
            gridEstimate = 2*pi*fullPitchGrid(idx);
            % refine the estimate (only refine if the grid is too coarse)
            freqSpacing = 2*pi*diff(fullPitchGrid(1:2));
            if accuracy < freqSpacing
                for lOrder = 1:maxModelOrder
                    initialLowerBound = gridEstimate(lOrder) - freqSpacing;
                    initialUpperBound = gridEstimate(lOrder) + freqSpacing;
                    objFunc = @(freq) -pitchCostFunction(freq, data, lOrder);
                    [finalLower, finalUpper] = fibonacciSearch(objFunc, ...
                        initialLowerBound, initialUpperBound, accuracy);
                    if lOrder == trueModelOrder
                        estimatedFreq = (finalUpper+finalLower)/2;
                    end
                end
            else
                estimatedFreq = gridEstimate(trueModelOrder);
            end
            computationTime(jMc,iSnr,kGridSize) = toc;
            estimationError(jMc,iSnr,kGridSize) = estimatedFreq-trueFreq;
        end
    end
end

%% Compute the average error and computation time
rmse = nan(nSnrs, nGridSizes);
minComputationTime = nan(nSnrs, nGridSizes);
for kGridSize = 1:nGridSizes
    rmse(:,kGridSize) = sqrt(mean(estimationError(:,:,kGridSize).^2))';
    minComputationTime(:,kGridSize) = min(computationTime(:,:,kGridSize))';
end
avgComputationTime = mean(minComputationTime);
%% plot the results

figure(1)
semilogy(snrList,[sqrt(crlb),rmse])
xlabel('snr [dB]')
ylabel('RMSE [cycles/sample]')
legend(['CRLB';cellstr(num2str(gainList', 'g = %-d'));'No refinement'])

%% store the data
comment = ['The RMSE vs SNR for various grid sizes. The gains are ', num2str(gainList, 'g = %-d ')];
rmseData = [snrList, sqrt(crlb), rmse];
labels = 'snr crlb gridA gridB gridC gridD gridE gridF fftOnly';
mtxToTblPgfplots('pitch',comment,labels,rmseData)
