function tests = singlePitchMlComplexCostFunctionTest()
%
% A matalb unittest. Can be executed as
%  
%   runtests('singlePitchMlComplexCostFunctionTest')
%
%
    tests = functiontests(localfunctions);
end    

function testProducedCostFunctionAlgorithm2(testCase)

    rng(100);
    nData = 10;
    modelOrder = 2;
    modelOrders = (1:modelOrder)';
    pitch = (1/modelOrder-0.5/nData)*rand(1)+0.5/nData;
    pitchBounds = [1/nData,1-1/nData];
    time = (0:nData-1)';
    Z = exp(1i*2*pi*pitch*time*modelOrders');
    amplitudes = randn(modelOrder,2)*[1;1i];
    data = Z*amplitudes;
    maxModelOrder = 5;
    resolution = 1/(nData*maxModelOrder*5);

    [expCostFunctionMatrix, ~] = ...
        singlePitchMlCostFunctionNaiveImplementation(data,...
                                                     maxModelOrder,resolution,pitchBounds);


    [Algorithm2CostFunctionMatrix] = ...
        singlePitchMlComplexCostFunctionAlgorithm2(data,...
                                                   maxModelOrder,resolution);

    nFftGrid = ceil(1/resolution);
    minFftIndex = ceil(nFftGrid*pitchBounds(1));
    maxFftIndex = floor(nFftGrid*pitchBounds(2));

    testCase.assertEqual(expCostFunctionMatrix,...
                         Algorithm2CostFunctionMatrix(:,minFftIndex+1:maxFftIndex+1),...
                         'relTol', 1e-10);


end

function testProducedCostFunction1(testCase)

    % Seeding is just used to make debuggin easier. Outcomment as
    % needed
    %rng(100);

    nData = 10;
    modelOrder = 1;
    modelOrders = (1:modelOrder)';
    pitch = (1/modelOrder-0.5/nData)*rand(1)+0.5/nData;
    pitchBounds = [1/nData,1-1/nData];
    time = (0:nData-1)';
    Z = exp(1i*2*pi*pitch*time*modelOrders');
    amplitudes = randn(modelOrder,2)*[1;1i];
    data = Z*amplitudes;
    maxModelOrder = 5;
    resolution = 1/(nData*maxModelOrder*5);

    [expCostFunctionMatrix, ~] = ...
        singlePitchMlCostFunctionNaiveImplementation(data,...
        maxModelOrder,resolution,pitchBounds);

    [expSlightlyCostFunctionMatrix, ~] = ...
        singlePitchMlCostFunctionSlightlyLessNaiveImplementation(data,...
        maxModelOrder,resolution,pitchBounds);

    [actCostFunctionMatrix, fullPitchGrid] = ...
        singlePitchMlComplexCostFunction(data,...
        maxModelOrder,resolution,pitchBounds);

    [actCostFunctionMatrixCpp, ~] = ...
        singlePitchMlComplexCostFunction(data,...
        maxModelOrder,resolution,pitchBounds,1);


    testCase.assertEqual(expSlightlyCostFunctionMatrix,...
                         expCostFunctionMatrix,...
                         'relTol',1e-10);

    testCase.assertEqual(actCostFunctionMatrix,expCostFunctionMatrix,...
        'relTol',1e-10);

    testCase.assertEqual(actCostFunctionMatrixCpp',expCostFunctionMatrix,...
                         'relTol',1e-10);

end


function testProducedCostFunction2(testCase)

    % Seeding is just used to make debuggin easier. Outcomment as
    % needed
    rng(100);

    nData = 10;
    modelOrder = 1;
    modelOrders = (1:modelOrder)';
    pitch = (1/modelOrder-0.5/nData)*rand(1)+0.5/nData;
    pitchBounds = [1/nData,1-1/nData];
    time = (0:nData-1)';
    Z = exp(1i*2*pi*pitch*time*modelOrders');
    amplitudes = randn(modelOrder,2)*[1;1i];
    data = Z*amplitudes;
    maxModelOrder = 5;
    resolution = 1/(nData*maxModelOrder*5);

    [expCostFunctionMatrix, ~] = ...
        singlePitchMlCostFunctionNaiveImplementation(data,...
        maxModelOrder,resolution,pitchBounds);

    [expSlightlyCostFunctionMatrix, ~] = ...
        singlePitchMlCostFunctionSlightlyLessNaiveImplementation(data,...
        maxModelOrder,resolution,pitchBounds);

    [actCostFunctionMatrix, fullPitchGrid] = ...
        singlePitchMlComplexCostFunction(data,...
        maxModelOrder,resolution,pitchBounds);

    [actCostFunctionMatrixCpp, ~] = ...
        singlePitchMlComplexCostFunction(data,...
        maxModelOrder,resolution,pitchBounds,1);

    testCase.assertEqual(expSlightlyCostFunctionMatrix,...
                         expCostFunctionMatrix,...
                         'relTol',1e-10);

    testCase.assertEqual(actCostFunctionMatrix,expCostFunctionMatrix,...
        'relTol',1e-10);

    testCase.assertEqual(actCostFunctionMatrixCpp',expCostFunctionMatrix,...
                         'relTol',1e-10);

    
end


function testProducedCostFunction3(testCase)

    % Seeding is just used to make debuggin easier. Outcomment as
    % needed
    rng(100);

    nData = 100;
    modelOrder = 30;
    modelOrders = (1:modelOrder)';
    pitch = (1/modelOrder-0.5/nData)*rand(1)+0.5/nData;
    pitchBounds = [1/nData,1-1/nData];
    time = (0:nData-1)';
    Z = exp(1i*2*pi*pitch*time*modelOrders');
    amplitudes = randn(modelOrder,2)*[1;1i];
    data = Z*amplitudes;
    maxModelOrder = 30;
    resolution = 1/(nData*maxModelOrder*5);

    [expCostFunctionMatrix, ~] = ...
        singlePitchMlCostFunctionNaiveImplementation(data,...
        maxModelOrder,resolution,pitchBounds);

    [expSlightlyCostFunctionMatrix, ~] = ...
        singlePitchMlCostFunctionSlightlyLessNaiveImplementation(data,...
        maxModelOrder,resolution,pitchBounds);

    [actCostFunctionMatrix, fullPitchGrid] = ...
        singlePitchMlComplexCostFunction(data,...
        maxModelOrder,resolution,pitchBounds);

    [actCostFunctionMatrixCpp, ~] = ...
        singlePitchMlComplexCostFunction(data,...
        maxModelOrder,resolution,pitchBounds,1);


    testCase.assertEqual(expSlightlyCostFunctionMatrix,...
                         expCostFunctionMatrix,...
                         'relTol',1e-10);

    testCase.assertEqual(actCostFunctionMatrix,expCostFunctionMatrix,...
        'relTol',1e-10);

    testCase.assertEqual(actCostFunctionMatrixCpp',expCostFunctionMatrix,...
                         'relTol',1e-10);

end

