function tests = dft2SymmetricDftTest()
    tests = functiontests(localfunctions);
end

% Test for an even number of samples
function testTransformationEvenSamples(testCase)
    nDataList = [2,4,10,12,50,100,200];
    nNDataList = length(nDataList);
    nSignals = 2;
    for iNData = 1:nNDataList
        nData = nDataList(iNData);
        signal = randn(nData,nSignals);
        dftSignal = fft(signal);
        symmetricDftSignal = dft2SymmetricDft(dftSignal);
        for iSignal = 1:nSignals
            % The middle element of symmetricDftSignal must be real-valued and
            % equal to the first element of dftSignal
            testCase.assertTrue(isreal(symmetricDftSignal(nData/2+1,iSignal)));
            testCase.assertEqual(symmetricDftSignal(nData/2+1,iSignal)*nData,...
                dftSignal(1,iSignal),'absTol',1e-12);
            % The first and last elements of symmetricDftSignal must be real-valued
            % and equal half of the (nData/2+1)'th element of dftSignal
            testCase.assertTrue(isreal(symmetricDftSignal(1,iSignal)));
            testCase.assertTrue(isreal(symmetricDftSignal(end,iSignal)));
            testCase.assertEqual(symmetricDftSignal(1,iSignal),...
                symmetricDftSignal(end,iSignal));
            testCase.assertEqual(2*symmetricDftSignal(1,iSignal)*nData,...
                dftSignal(nData/2+1,iSignal),'absTol',1e-12);
            % Check that our symmetric DFT representation of the signal is the same
            % as the signal.
            reconstructedSignal = ...
                exp(1i*2*pi*(0:nData-1)'*(-nData/2:nData/2)/nData)*...
                symmetricDftSignal(:,iSignal);
            testCase.assertEqual(reconstructedSignal,signal(:,iSignal),...
                'absTol',1e-12);
            % Finally, symmetricDftSignal must be conjugate symmetric
            testCase.assertEqual(symmetricDftSignal(:,iSignal),...
                conj(flipud(symmetricDftSignal(:,iSignal))));
        end
    end
end

% Test for an uneven number of samples
function testTransformationUnevenSamples(testCase)
    nDataList = [1,3,11,13,51,101,201];
    nNDataList = length(nDataList);
    nSignals = 2;
    for iNData = 1:nNDataList
        nData = nDataList(iNData);
        signal = randn(nData,nSignals);
        dftSignal = fft(signal,[],1);
        symmetricDftSignal = dft2SymmetricDft(dftSignal);
        for iSignal = 1:nSignals
            % The middle element of symmetricDftSignal must be real-valued and
            % equal to the first element of dftSignal
            testCase.assertTrue(isreal(symmetricDftSignal((nData+1)/2,iSignal)));
            testCase.assertEqual(symmetricDftSignal((nData+1)/2,iSignal)*nData,...
                dftSignal(1,iSignal),'absTol',1e-12);
            % Check that our symmetric DFT representation of the signal is the same
            % as the signal.
            reconstructedSignal = ...
                exp(1i*2*pi*(0:nData-1)'*(-floor(nData/2):floor(nData/2))/nData)*...
                symmetricDftSignal(:,iSignal);
            testCase.assertEqual(reconstructedSignal,signal(:,iSignal),...
                'absTol',1e-12);
            % Finally, symmetricDftSignal must be conjugate symmetric
            testCase.assertEqual(symmetricDftSignal(:,iSignal),...
                conj(flipud(symmetricDftSignal(:,iSignal))));
        end
    end
end
